@@ -2,7 +2,7 @@
 /*
  *  Authors: Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2002 Ximian, Inc. (www.ximian.com)
+ *  Copyright 2002-2004 Ximian, Inc. (www.ximian.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -33,6 +33,12 @@
 #include "gmime-iconv-utils.h"
 #include "gmime-charset.h"
 
+#ifdef ENABLE_WARNINGS
+#define w(x) x
+#else
+#define w(x)
+#endif /* ENABLE_WARNINGS */
+
 #ifdef G_THREADS_ENABLED
 static GStaticMutex lock = G_STATIC_MUTEX_INIT;
 #define LOCK()   g_static_mutex_lock (&lock)
@@ -57,8 +63,8 @@ iconv_utils_init (void)
 	
 	g_mime_charset_map_init ();
 	
-	utf8 = g_mime_charset_name ("utf-8");
-	locale = g_mime_charset_name (g_mime_charset_locale_name ());
+	utf8 = g_mime_charset_iconv_name ("utf-8");
+	locale = g_mime_charset_iconv_name (g_mime_locale_charset ());
 	
 	utf8_to_locale = iconv_open (locale, utf8);
 	locale_to_utf8 = iconv_open (utf8, locale);
@@ -148,7 +154,7 @@ g_mime_iconv_strndup (iconv_t cd, const char *string, size_t n)
 	
  fail:
 	
-	g_warning ("g_mime_iconv_strndup: %s", g_strerror (errno));
+	w(g_warning ("g_mime_iconv_strndup: %s", strerror (errno)));
 	
 	g_free (out);
 	
