@@ -107,7 +107,7 @@ g_mime_message_part_finalize (GObject *object)
 	GMimeMessagePart *part = (GMimeMessagePart *) object;
 	
 	if (part->message)
-		g_mime_object_unref (GMIME_OBJECT (part->message));
+		g_object_unref (part->message);
 	
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -186,23 +186,20 @@ message_part_write_to_stream (GMimeObject *object, GMimeStream *stream)
 	ssize_t nwritten, total = 0;
 	
 	/* write the content headers */
-	nwritten = g_mime_header_write_to_stream (object->headers, stream);
-	if (nwritten == -1)
+	if ((nwritten = g_mime_header_write_to_stream (object->headers, stream)) == -1)
 		return -1;
 	
 	total += nwritten;
 	
 	/* terminate the headers */
-	nwritten = g_mime_stream_write (stream, "\n", 1);
-	if (nwritten == -1)
+	if ((nwritten = g_mime_stream_write (stream, "\n", 1)) == -1)
 		return -1;
 	
 	total += nwritten;
 	
 	/* write the message */
 	if (part->message) {
-		nwritten = g_mime_object_write_to_stream (GMIME_OBJECT (part->message), stream);
-		if (nwritten == -1)
+		if ((nwritten = g_mime_object_write_to_stream (GMIME_OBJECT (part->message), stream)) == -1)
 			return -1;
 		
 		total += nwritten;
@@ -255,7 +252,7 @@ g_mime_message_part_new_with_message (const char *subtype, GMimeMessage *message
 	
 	part = g_mime_message_part_new (subtype);
 	part->message = message;
-	g_mime_object_ref (GMIME_OBJECT (message));
+	g_object_ref (message);
 	
 	return part;
 }
@@ -274,10 +271,10 @@ g_mime_message_part_set_message (GMimeMessagePart *part, GMimeMessage *message)
 	g_return_if_fail (GMIME_IS_MESSAGE_PART (part));
 	
 	if (message)
-		g_mime_object_ref (GMIME_OBJECT (message));
+		g_object_ref (message);
 	
 	if (part->message)
-		g_mime_object_unref (GMIME_OBJECT (part->message));
+		g_object_unref (part->message);
 	
 	part->message = message;
 }
