@@ -25,6 +25,7 @@
 #include <config.h>
 #endif
 
+#include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
 #include <regex.h>
@@ -99,6 +100,11 @@ struct _GMimeParserPrivate {
 	unsigned char *headerptr;
 	unsigned int headerleft;
 	
+	/* raw header buffer */
+	unsigned char *rawbuf;
+	unsigned char *rawptr;
+	unsigned int rawleft;
+	
 	off_t headers_start;
 	off_t header_start;
 	
@@ -119,18 +125,19 @@ struct _GMimeParserPrivate {
 struct _boundary_stack {
 	struct _boundary_stack *parent;
 	unsigned char *boundary;
-	unsigned int boundarylen;
-	unsigned int boundarylenfinal;
-	unsigned int boundarylenmax;
+	size_t boundarylen;
+	size_t boundarylenfinal;
+	size_t boundarylenmax;
 	off_t content_end;
 };
 
+
 static void
 parser_push_boundary (GMimeParser *parser, const char *boundary)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
 	struct _boundary_stack *s;
-	unsigned int max;
+	size_t max;
 	
 	max = priv->bounds ? priv->bounds->boundarylenmax : 0;
 	
@@ -258,6 +265,7 @@ g_mime_parser_init (GMimeParser *parser, GMimeParserClass *klass)
 	parser->priv->scan_from = FALSE;
 	parser->priv->have_regex = FALSE;
 	parser->priv->persist_stream = TRUE;
+	parser->priv->respect_content_length = FALSE;
 }
 
 static void
@@ -283,7 +291,7 @@ parser_init (GMimeParser *parser, GMimeStream *stream)
 	off_t offset = -1;
 	
 	if (stream) {
-		g_mime_stream_ref (stream);
+		g_object_ref (stream);
 		offset = g_mime_stream_tell (stream);
 	}
 	
@@ -304,6 +312,10 @@ parser_init (GMimeParser *parser, GMimeStream *stream)
 	priv->headerptr = priv->headerbuf;
 	priv->headerleft = SCAN_HEAD;
 	
+	priv->rawbuf = g_malloc (SCAN_HEAD + 1);
+	priv->rawptr = priv->rawbuf;
+	priv->rawleft = SCAN_HEAD;
+	
 	priv->headers_start = -1;
 	priv->header_start = -1;
 	
@@ -322,12 +334,12 @@ parser_close (GMimeParser *parser)
 	struct _GMimeParserPrivate *priv = parser->priv;
 	
 	if (priv->stream)
-		g_mime_stream_unref (priv->stream);
+		g_object_unref (priv->stream);
 	
 	g_byte_array_free (priv->from_line, TRUE);
 	
-	if (priv->headerbuf)
-		g_free (priv->headerbuf);
+	g_free (priv->headerbuf);
+	g_free (priv->rawbuf);
 	
 	header_raw_clear (&priv->headers);
 	
@@ -356,7 +368,7 @@ g_mime_parser_new (void)
 {
 	GMimeParser *parser;
 	
-	parser = g_object_new (GMIME_TYPE_PARSER, NULL, NULL);
+	parser = g_object_new (GMIME_TYPE_PARSER, NULL);
 	
 	return parser;
 }
@@ -397,11 +409,11 @@ g_mime_parser_new_with_stream (GMimeStream *stream)
  * or resetting of the stream. Anything that will/could change the
  * current stream's offset is PROHIBITED.
  *
- * It is also recommended that you not use #g_mime_stream_tell because
- * it will not necessarily give you the current @parser offset since
- * @parser handles its own internal read-ahead buffer. Instead, it is
- * recommended that you use #g_mime_parser_tell if you have a reason
- * to need the current offset of the @parser.
+ * It is also recommended that you not use g_mime_stream_tell()
+ * because it will not necessarily give you the current @parser offset
+ * since @parser handles its own internal read-ahead buffer. Instead,
+ * it is recommended that you use g_mime_parser_tell() if you have a
+ * reason to need the current offset of the @parser.
  **/
 void
 g_mime_parser_init_with_stream (GMimeParser *parser, GMimeStream *stream)
@@ -513,9 +525,9 @@ g_mime_parser_get_respect_content_length (GMimeParser *parser)
 
 
 /**
- * g_mime_parser_get_respect_content_length:
+ * g_mime_parser_set_respect_content_length:
  * @parser: MIME parser object
- * @use_content_length: %TRUE if the parser should use Content-Length headers or %FALSE otherwise.
+ * @respect_content_length: %TRUE if the parser should use Content-Length headers or %FALSE otherwise.
  *
  * Sets whether or not @parser should respect Content-Length headers
  * when deciding where to look for the start of the next message. Only
@@ -574,8 +586,8 @@ parser_fill (GMimeParser *parser)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
 	unsigned char *inbuf, *inptr, *inend;
-	size_t inlen, atleast = SCAN_HEAD;
 	ssize_t nread;
+	size_t inlen;
 	
 	inbuf = priv->inbuf;
 	inptr = priv->inptr;
@@ -584,11 +596,6 @@ parser_fill (GMimeParser *parser)
 	
 	g_assert (inptr <= inend);
 	
-	atleast = MAX (atleast, priv->bounds ? priv->bounds->boundarylenmax : 0);
-	
-	if (inlen > atleast)
-		return inlen;
-	
 	/* attempt to align 'inend' with realbuf + SCAN_HEAD */
 	if (inptr >= inbuf) {
 		inbuf -= inlen < SCAN_HEAD ? inlen : SCAN_HEAD;
@@ -611,8 +618,7 @@ parser_fill (GMimeParser *parser)
 	priv->inend = inbuf;
 	inend = priv->realbuf + SCAN_HEAD + SCAN_BUF - 1;
 	
-	nread = g_mime_stream_read (priv->stream, inbuf, inend - inbuf);
-	if (nread > 0)
+	if ((nread = g_mime_stream_read (priv->stream, inbuf, inend - inbuf)) > 0)
 		priv->inend += nread;
 	
 	priv->offset = g_mime_stream_tell (priv->stream);
@@ -624,6 +630,9 @@ parser_fill (GMimeParser *parser)
 static off_t
 parser_offset (struct _GMimeParserPrivate *priv, unsigned char *inptr)
 {
+	if (priv->offset == -1)
+		return -1;
+	
 	if (!inptr)
 		inptr = priv->inptr;
 	
@@ -725,9 +734,9 @@ parser_step_from (GMimeParser *parser)
 	return 0;
 }
 
-#define header_backup(priv, start, len) G_STMT_START {                    \
+#define header_append(priv, start, len) G_STMT_START {                    \
 	if (priv->headerleft <= len) {                                    \
-		unsigned int hlen, hoff;                                  \
+		size_t hlen, hoff;                                        \
 		                                                          \
 		hlen = hoff = priv->headerptr - priv->headerbuf;          \
 		hlen = hlen ? hlen : 1;                                   \
@@ -745,10 +754,35 @@ parser_step_from (GMimeParser *parser)
 	priv->headerleft -= len;                                          \
 } G_STMT_END
 
+#define raw_header_append(priv, start, len) G_STMT_START {                \
+	if (priv->rawleft <= len) {                                       \
+		size_t hlen, hoff;                                        \
+		                                                          \
+		hlen = hoff = priv->rawptr - priv->rawbuf;                \
+		hlen = hlen ? hlen : 1;                                   \
+		                                                          \
+		while (hlen < hoff + len)                                 \
+			hlen <<= 1;                                       \
+		                                                          \
+		priv->rawbuf = g_realloc (priv->rawbuf, hlen + 1);        \
+		priv->rawptr = priv->rawbuf + hoff;                       \
+		priv->rawleft = hlen - hoff;                              \
+	}                                                                 \
+	                                                                  \
+	memcpy (priv->rawptr, start, len);                                \
+	priv->rawptr += len;                                              \
+	priv->rawleft -= len;                                             \
+} G_STMT_END
+
+#define raw_header_reset(priv) G_STMT_START {                             \
+	priv->rawleft += priv->rawptr - priv->rawbuf;                     \
+	priv->rawptr = priv->rawbuf;                                      \
+} G_STMT_END
+
 #define header_parse(parser, priv, hend) G_STMT_START {                   \
 	register unsigned char *colon;                                    \
 	struct _header_raw *header;                                       \
-	unsigned int hlen;                                                \
+	size_t hlen;                                                      \
 	                                                                  \
 	header = g_new (struct _header_raw, 1);                           \
 	header->next = NULL;                                              \
@@ -819,23 +853,29 @@ parser_step_headers (GMimeParser *parser)
 			while (*inptr != '\n')
 				inptr++;
 			
+			raw_header_append (priv, start, inptr - start);
+			
 			if (inptr == inend) {
 				/* we don't have enough data to tell if we
 				   got all of the header or not... */
-				priv->inptr = start;
-				len = inend - start;
+				priv->inptr = inptr;
+				len = inend - inptr;
 				goto refill;
 			}
 			
 			/* check to see if we've reached the end of the headers */
-			if (!priv->midline && inptr == start)
+			if (!priv->midline && (inptr == start || (inptr - start == 1 && *start == '\r')))
 				goto headers_end;
 			
 			len = inptr - start;
-			header_backup (priv, start, len);
+			if (inptr[-1] == '\r')
+				len--;
+			
+			header_append (priv, start, len);
 			
 			if (inptr < inend) {
 				/* inptr has to be less than inend - 1 */
+				raw_header_append (priv, inptr, 1);
 				inptr++;
 				if (*inptr == ' ' || *inptr == '\t') {
 					priv->midline = TRUE;
@@ -856,13 +896,15 @@ parser_step_headers (GMimeParser *parser)
 	inptr = priv->inptr;
 	inend = priv->inend;
 	
-	header_backup (priv, inptr, inend - inptr);
+	header_append (priv, inptr, inend - inptr);
 	
  headers_end:
 	
 	if (priv->headerptr > priv->headerbuf)
 		header_parse (parser, priv, hend);
 	
+	*priv->rawptr = '\0';
+	
 	priv->state = GMIME_PARSER_STATE_HEADERS_END;
 	
 	g_assert (inptr <= priv->inend);
@@ -878,8 +920,7 @@ parser_content_type (GMimeParser *parser)
 	struct _GMimeParserPrivate *priv = parser->priv;
 	const char *content_type;
 	
-	content_type = header_raw_find (priv->headers, "Content-Type", NULL);
-	if (content_type)
+	if ((content_type = header_raw_find (priv->headers, "Content-Type", NULL)))
 		return g_mime_content_type_new_from_string (content_type);
 	
 	return NULL;
@@ -966,10 +1007,13 @@ enum {
 			  (len >= 2 && (start[0] == '-' && start[1] == '-')))
 
 static int
-check_boundary (struct _GMimeParserPrivate *priv, const unsigned char *start, int len)
+check_boundary (struct _GMimeParserPrivate *priv, const unsigned char *start, size_t len)
 {
 	off_t offset = parser_offset (priv, (unsigned char *) start);
 	
+	if (start[len - 1] == '\r')
+		len--;
+	
 	if (possible_boundary (priv->scan_from, start, len)) {
 		struct _boundary_stack *s;
 		
@@ -1014,7 +1058,7 @@ check_boundary (struct _GMimeParserPrivate *priv, const unsigned char *start, in
  **/
 
 static int
-parser_scan_content (GMimeParser *parser, GByteArray *content)
+parser_scan_content (GMimeParser *parser, GByteArray *content, int *crlf)
 {
 	struct _GMimeParserPrivate *priv = parser->priv;
 	register unsigned char *inptr;
@@ -1058,9 +1102,10 @@ parser_scan_content (GMimeParser *parser, GByteArray *content)
 			len = inptr - start;
 			
 			if (inptr < inend) {
-				inptr++;
 				if ((found = check_boundary (priv, start, len)))
 					goto boundary;
+				
+				inptr++;
 				len++;
 			} else {
 				/* didn't find an end-of-line */
@@ -1089,6 +1134,15 @@ parser_scan_content (GMimeParser *parser, GByteArray *content)
 	/* don't chew up the boundary */
 	priv->inptr = start;
 	
+	if (found != FOUND_EOS) {
+		if (inptr[-1] == '\r')
+			*crlf = 2;
+		else
+			*crlf = 1;
+	} else {
+		*crlf = 0;
+	}
+	
 	return found;
 }
 
@@ -1101,19 +1155,20 @@ parser_scan_mime_part_content (GMimeParser *parser, GMimePart *mime_part, int *f
 	GMimeDataWrapper *wrapper;
 	GMimeStream *stream;
 	off_t start, end;
+	int crlf;
 	
 	if (priv->persist_stream && priv->seekable)
 		start = parser_offset (priv, NULL);
 	else
 		content = g_byte_array_new ();
 	
-	*found = parser_scan_content (parser, content);
+	*found = parser_scan_content (parser, content, &crlf);
 	if (*found != FOUND_EOS) {
 		/* last '\n' belongs to the boundary */
 		if (priv->persist_stream && priv->seekable)
-			end = parser_offset (priv, NULL) - 1;
+			end = parser_offset (priv, NULL) - crlf;
 		else
-			g_byte_array_set_size (content, MAX (content->len - 1, 0));
+			g_byte_array_set_size (content, MAX (content->len - crlf, 0));
 	} else if (priv->persist_stream && priv->seekable) {
 		end = parser_offset (priv, NULL);
 	}
@@ -1128,8 +1183,8 @@ parser_scan_mime_part_content (GMimeParser *parser, GMimePart *mime_part, int *f
 	
 	wrapper = g_mime_data_wrapper_new_with_stream (stream, encoding);
 	g_mime_part_set_content_object (mime_part, wrapper);
-	g_mime_stream_unref (stream);
 	g_object_unref (wrapper);
+	g_object_unref (stream);
 }
 
 static void
@@ -1149,26 +1204,24 @@ parser_scan_message_part (GMimeParser *parser, GMimeMessagePart *mpart, int *fou
 	message = g_mime_message_new (FALSE);
 	header = priv->headers;
 	while (header) {
-		g_mime_object_add_header (GMIME_OBJECT (message), header->name, header->value);
+		g_mime_object_add_header ((GMimeObject *) message, header->name, header->value);
 		header = header->next;
 	}
 	
-	content_type = parser_content_type (parser);
-	if (!content_type)
+	if (!(content_type = parser_content_type (parser)))
 		content_type = g_mime_content_type_new ("text", "plain");
 	
 	parser_unstep (parser);
-	if (content_type && g_mime_content_type_is_type (content_type, "multipart", "*")) {
+	if (g_mime_content_type_is_type (content_type, "multipart", "*")) {
 		object = parser_construct_multipart (parser, content_type, found);
 	} else {
 		object = parser_construct_leaf_part (parser, content_type, found);
 	}
 	
-	g_mime_message_set_mime_part (message, object);
-	g_mime_object_unref (object);
+	message->mime_part = object;
 	
 	g_mime_message_part_set_message (mpart, message);
-	g_mime_object_unref (GMIME_OBJECT (message));
+	g_object_unref (message);
 }
 
 static GMimeObject *
@@ -1182,12 +1235,6 @@ parser_construct_leaf_part (GMimeParser *parser, GMimeContentType *content_type,
 	while (parser_step (parser) != GMIME_PARSER_STATE_HEADERS_END)
 		;
 	
-	if (!content_type) {
-		content_type = parser_content_type (parser);
-		if (!content_type)
-			content_type = g_mime_content_type_new ("text", "plain");
-	}
-	
 	object = g_mime_object_new_type (content_type->type, content_type->subtype);
 	header = priv->headers;
 	while (header) {
@@ -1197,7 +1244,12 @@ parser_construct_leaf_part (GMimeParser *parser, GMimeContentType *content_type,
 	
 	header_raw_clear (&priv->headers);
 	
-	g_mime_object_set_content_type (object, content_type);
+	if (object->content_type)
+		g_mime_content_type_destroy (object->content_type);
+	object->content_type = content_type;
+	
+	g_mime_header_set_raw (object->headers, priv->rawbuf);
+	raw_header_reset (priv);
 	
 	/* skip empty line after headers */
 	parser_skip_line (parser);
@@ -1211,27 +1263,53 @@ parser_construct_leaf_part (GMimeParser *parser, GMimeContentType *content_type,
 	return object;
 }
 
+static void
+crlf2lf (unsigned char *in)
+{
+	register unsigned char *inptr = in;
+	register unsigned char *outptr;
+	
+	while (*inptr != '\0' && !(inptr[0] == '\r' && inptr[1] == '\n'))
+		inptr++;
+	
+	if (*inptr == '\0')
+		return;
+	
+	outptr = inptr++;
+	
+	while (*inptr != '\0') {
+		while (*inptr != '\0' && !(inptr[0] == '\r' && inptr[1] == '\n'))
+			*outptr++ = *inptr++;
+		
+		if (*inptr == '\r')
+			inptr++;
+	}
+	
+	*outptr = '\0';
+}
+
 static int
 parser_scan_multipart_face (GMimeParser *parser, GMimeMultipart *multipart, gboolean preface)
 {
 	GByteArray *buffer;
 	const char *face;
-	int found;
+	int crlf, found;
 	
 	buffer = g_byte_array_new ();
-	found = parser_scan_content (parser, buffer);
+	found = parser_scan_content (parser, buffer, &crlf);
 	
-	/* last '\n' belongs to the boundary */
-	if (buffer->len) {
-		buffer->data[buffer->len - 1] = '\0';
+	if (buffer->len >= crlf) {
+		/* last '\n' belongs to the boundary */
+		g_byte_array_set_size (buffer, buffer->len + 1);
+		buffer->data[buffer->len - crlf - 1] = '\0';
+		crlf2lf (buffer->data);
 		face = buffer->data;
-	} else
-		face = NULL;
-	
-	if (preface)
-		g_mime_multipart_set_preface (multipart, face);
-	else
-		g_mime_multipart_set_postface (multipart, face);
+		
+		if (preface)
+			g_mime_multipart_set_preface (multipart, face);
+		else
+			g_mime_multipart_set_postface (multipart, face);
+	}
 	
 	g_byte_array_free (buffer, TRUE);
 	
@@ -1255,8 +1333,7 @@ parser_scan_multipart_subparts (GMimeParser *parser, GMimeMultipart *multipart)
 		/* get the headers */
 		parser_step_headers (parser);
 		
-		content_type = parser_content_type (parser);
-		if (!content_type)
+		if (!(content_type = parser_content_type (parser)))
 			content_type = g_mime_content_type_new ("text", "plain");
 		
 		parser_unstep (parser);
@@ -1267,7 +1344,7 @@ parser_scan_multipart_subparts (GMimeParser *parser, GMimeMultipart *multipart)
 		}
 		
 		g_mime_multipart_add_part (multipart, subpart);
-		g_mime_object_unref (subpart);
+		g_object_unref (subpart);
 	} while (found == FOUND_BOUNDARY);
 	
 	return found;
@@ -1303,7 +1380,12 @@ parser_construct_multipart (GMimeParser *parser, GMimeContentType *content_type,
 	
 	header_raw_clear (&priv->headers);
 	
-	g_mime_object_set_content_type (object, content_type);
+	if (object->content_type)
+		g_mime_content_type_destroy (object->content_type);
+	object->content_type = content_type;
+	
+	g_mime_header_set_raw (object->headers, priv->rawbuf);
+	raw_header_reset (priv);
 	
 	multipart = (GMimeMultipart *) object;
 	
@@ -1322,10 +1404,11 @@ parser_construct_multipart (GMimeParser *parser, GMimeContentType *content_type,
 		if (*found == FOUND_END_BOUNDARY && found_immediate_boundary (priv)) {
 			/* eat end boundary */
 			parser_skip_line (parser);
+			parser_pop_boundary (parser);
 			*found = parser_scan_multipart_postface (parser, multipart);
+		} else {
+			parser_pop_boundary (parser);
 		}
-		
-		parser_pop_boundary (parser);
 	} else {
 		w(g_warning ("multipart without boundary encountered"));
 		/* this will scan everything into the preface */
@@ -1346,8 +1429,7 @@ parser_construct_part (GMimeParser *parser)
 	while (parser_step (parser) != GMIME_PARSER_STATE_HEADERS_END)
 		;
 	
-	content_type = parser_content_type (parser);
-	if (!content_type)
+	if (!(content_type = parser_content_type (parser)))
 		content_type = g_mime_content_type_new ("text", "plain");
 	
 	parser_unstep (parser);
@@ -1399,7 +1481,7 @@ parser_construct_message (GMimeParser *parser)
 		if (priv->respect_content_length && !strcasecmp (header->name, "Content-Length"))
 			content_length = strtoul (header->value, NULL, 10);
 		
-		g_mime_object_add_header (GMIME_OBJECT (message), header->name, header->value);
+		g_mime_object_add_header ((GMimeObject *) message, header->name, header->value);
 		header = header->next;
 	}
 	
@@ -1409,8 +1491,7 @@ parser_construct_message (GMimeParser *parser)
 			priv->bounds->content_end = parser_offset (priv, NULL) + content_length;
 	}
 	
-	content_type = parser_content_type (parser);
-	if (!content_type)
+	if (!(content_type = parser_content_type (parser)))
 		content_type = g_mime_content_type_new ("text", "plain");
 	
 	parser_unstep (parser);
@@ -1420,8 +1501,7 @@ parser_construct_message (GMimeParser *parser)
 		object = parser_construct_leaf_part (parser, content_type, &found);
 	}
 	
-	g_mime_message_set_mime_part (message, object);
-	g_mime_object_unref (object);
+	message->mime_part = object;
 	
 	if (priv->scan_from) {
 		priv->state = GMIME_PARSER_STATE_FROM;
@@ -1454,7 +1534,7 @@ g_mime_parser_construct_message (GMimeParser *parser)
  * @parser: MIME parser object
  *
  * Gets the mbox-style From-line of the most recently parsed message
- * (gotten from #g_mime_parser_construct_message).
+ * (gotten from g_mime_parser_construct_message()).
  *
  * Returns the mbox-style From-line of the most recently parsed
  * message or %NULL on error.
@@ -1482,7 +1562,7 @@ g_mime_parser_get_from (GMimeParser *parser)
  * @parser: MIME parser object
  *
  * Gets the offset of the most recently parsed mbox-style From-line
- * (gotten from #g_mime_parser_construct_message).
+ * (gotten from g_mime_parser_construct_message()).
  *
  * Returns the offset of the most recently parsed mbox-style From-line
  * or -1 on error.
