@@ -1,9 +1,9 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- *  Authors: Michael Zucchi <notzed@helixcode.com>
- *           Jeffrey Stedfast <fejj@helixcode.com>
+ *  Authors: Michael Zucchi <notzed@ximian.com>
+ *           Jeffrey Stedfast <fejj@ximian.com>
  *
- *  Copyright 2000 Helix Code, Inc. (www.helixcode.com)
+ *  Copyright 2000-2004 Ximian, Inc. (www.ximian.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -21,18 +21,19 @@
  *
  */
 
+
 #ifndef __GMIME_UTILS_H__
 #define __GMIME_UTILS_H__
 
+#include <glib.h>
+#include <time.h>
+#include <stdarg.h>
+
 #ifdef __cplusplus
 extern "C" {
 #pragma }
 #endif /* __cplusplus */
 
-#include <glib.h>
-#include <time.h>
-#include <stdarg.h>
-
 typedef enum {
 	GMIME_PART_ENCODING_DEFAULT,
 	GMIME_PART_ENCODING_7BIT,
@@ -72,6 +73,8 @@ GMimeReferences *g_mime_references_decode (const char *text);
 void g_mime_references_append (GMimeReferences **refs, const char *msgid);
 void g_mime_references_clear (GMimeReferences **refs);
 
+char  *g_mime_utils_structured_header_fold (const char *in);
+char  *g_mime_utils_unstructured_header_fold (const char *in);
 char  *g_mime_utils_header_fold (const char *in);
 char  *g_mime_utils_header_printf (const char *format, ...);
 
